"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllChildrenDirPathMap = exports.getBuildConfig = exports.getDevWebpackConfigByParams = exports.getProdWebpackConfigByParams = exports.getLibWebpackConfig = void 0;
const webpack_1 = require("webpack");
const path_1 = __importDefault(require("path"));
const mini_css_extract_plugin_1 = __importDefault(require("mini-css-extract-plugin"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const util_1 = require("../../util");
const sass_1 = __importDefault(require("sass"));
const postcssPxtorem = require('postcss-pxtorem');
const webpackModuleConfig = {
    rules: [
        {
            test: /\.(js|jsx)$/,
            loader: 'babel-loader',
            options: {
                compact: false,
                presets: [
                    [
                        '@babel/preset-env',
                        {
                            targets: {
                                chrome: '58',
                            },
                        },
                    ],
                    '@babel/preset-react',
                ],
                plugins: [
                    '@babel/plugin-proposal-class-properties',
                    ['@babel/plugin-proposal-decorators', { legacy: true }],
                    '@babel/plugin-proposal-export-default-from',
                    '@babel/plugin-proposal-export-namespace-from',
                    '@babel/plugin-proposal-optional-chaining',
                    '@babel/plugin-proposal-partial-application',
                    ['@babel/plugin-proposal-pipeline-operator', { proposal: 'minimal' }],
                ],
            },
        },
        {
            test: /\.(scss|sass)$/,
            use: [
                {
                    loader: mini_css_extract_plugin_1.default.loader,
                },
                {
                    loader: 'css-loader',
                },
                {
                    loader: 'postcss-loader',
                    options: {
                        ident: 'postcss',
                        plugins: () => [
                            postcssPxtorem({
                                rootValue: 14,
                                propList: ['*'],
                            }),
                        ],
                    },
                },
                {
                    loader: 'sass-loader',
                    options: {
                        implementation: sass_1.default,
                    },
                },
            ],
        },
        {
            test: /\.(less|css)$/,
            use: [
                {
                    loader: mini_css_extract_plugin_1.default.loader,
                },
                {
                    loader: 'css-loader',
                },
                {
                    loader: 'postcss-loader',
                    options: {
                        ident: 'postcss',
                        plugins: () => [
                            postcssPxtorem({
                                rootValue: 14,
                                propList: ['*'],
                            }),
                        ],
                    },
                },
                {
                    loader: 'less-loader',
                },
            ],
        },
        {
            test: /\.less$/,
            use: [
                {
                    loader: path_1.default.resolve(__dirname, './lessMergeLoader'),
                },
            ],
        },
        {
            test: /\.(jpe?g|png|gif|ttf|eot|svg|woff(2)?)(\?[a-z0-9=&.]+)?$/,
            loader: 'base64-inline-loader',
        },
    ],
};
const webpackResolveExtensions = ['.js', '.jsx', '.tsx', '.ts', '.json', '.scss', '.less', '.css'];
function getLibWebpackConfig(buildConfig) {
    const { name, outputPath, rootPath, mainEntryPath, controlEntryPath, mode } = buildConfig;
    (0, util_1.info)('生成 webpack 配置');
    const paramsConfig = {
        name,
        context: rootPath,
        entry: {
            main: mainEntryPath,
            control: controlEntryPath,
        },
        externals: {
            react: 'React',
            antd: 'Antd',
            'react-dom': 'ReactDom',
            'react-dom/server': 'ReactDomServer',
            '@formily/react-schema-renderer': 'formilyReactSchemaRenderer',
            '@formily/react': 'formilyReact',
            '@formily/core': 'formilyCore',
            '@formily/antd': 'formilyAntd',
        },
        output: {
            pathinfo: true,
            path: outputPath,
            filename: mode === 'development' ? `${name}.[name].js` : `${name}.[name].[hash].js`,
            library: `@weapps-materials-[name]-${name}`,
            libraryTarget: 'window',
            umdNamedDefine: true,
            publicPath: mode === 'development' ? '/static' : '',
        },
        resolve: {
            extensions: webpackResolveExtensions,
        },
        resolveLoader: {
            modules: [path_1.default.resolve(__dirname, '../../../node_modules'), path_1.default.resolve(rootPath, './node_modules')],
        },
    };
    if (mode === 'development') {
        return getDevWebpackConfigByParams(paramsConfig);
    }
    return getProdWebpackConfigByParams(paramsConfig);
}
exports.getLibWebpackConfig = getLibWebpackConfig;
function getProdWebpackConfigByParams({ context, entry, output, externals, resolve, resolveLoader, name, }) {
    return {
        name,
        context,
        entry,
        output,
        externals,
        resolve,
        resolveLoader,
        module: webpackModuleConfig,
        plugins: [
            new mini_css_extract_plugin_1.default({
                filename: `${name}.[name].[hash].css`,
            }),
        ],
        mode: 'production',
        optimization: {
            minimize: true,
        },
    };
}
exports.getProdWebpackConfigByParams = getProdWebpackConfigByParams;
function getDevWebpackConfigByParams({ context, entry, output, externals, resolve, resolveLoader, name }) {
    return {
        name,
        context,
        entry,
        output,
        externals,
        resolve,
        resolveLoader,
        module: webpackModuleConfig,
        plugins: [
            new mini_css_extract_plugin_1.default({
                filename: `${name}.[name].css`,
            }),
            new webpack_1.HotModuleReplacementPlugin(),
        ],
        mode: 'development',
        optimization: {
            minimize: false,
        },
        devtool: 'inline-source-map',
    };
}
exports.getDevWebpackConfigByParams = getDevWebpackConfigByParams;
function getBuildConfig(rootPath, mode = 'production') {
    (0, util_1.info)('初始化 build 配置');
    const outputPath = path_1.default.resolve(rootPath, './dist');
    const tempPath = path_1.default.resolve(rootPath, './.temp');
    const srcPath = path_1.default.resolve(rootPath, './src');
    if (!fs_extra_1.default.existsSync(tempPath)) {
        fs_extra_1.default.mkdirSync(tempPath);
    }
    const mainEntryPath = path_1.default.resolve(tempPath, './mainEntry.js');
    const controlEntryPath = path_1.default.resolve(tempPath, './controlEntry.js');
    const lessEntryPath = path_1.default.resolve(outputPath, './lessEntry.less');
    const manifestPath = path_1.default.resolve(outputPath, './manifest.json');
    const { name, version, title, description } = fs_extra_1.default.readJSONSync(path_1.default.resolve(rootPath, 'package.json'));
    const componentDirPathMap = getAllChildrenDirPathMap(srcPath, 'components');
    const pluginsDirPathMap = getAllChildrenDirPathMap(srcPath, 'plugins');
    const actionsDirPathMap = getAllChildrenDirPathMap(srcPath, 'actions');
    return {
        mode,
        name,
        version,
        description,
        title,
        rootPath,
        srcPath,
        tempPath,
        outputPath,
        mainEntryPath,
        controlEntryPath,
        lessEntryPath,
        manifestPath,
        componentDirPathMap,
        pluginsDirPathMap,
        actionsDirPathMap,
    };
}
exports.getBuildConfig = getBuildConfig;
function getAllChildrenDirPathMap(srcPath, dir) {
    const targetPath = path_1.default.join(srcPath, dir);
    if (!fs_extra_1.default.existsSync(targetPath)) {
        (0, util_1.warn)(`该项目下暂无 ${dir} 目录`);
        return {};
    }
    const map = fs_extra_1.default.readdirSync(targetPath).reduce((pathMap, name) => {
        const folder = path_1.default.join(targetPath, name.toString());
        if (fs_extra_1.default.lstatSync(folder).isDirectory()) {
            pathMap[name] = folder;
        }
        return pathMap;
    }, {});
    return map;
}
exports.getAllChildrenDirPathMap = getAllChildrenDirPathMap;
